<?php

// Check if WooCommerce is active
function check_woocommerce_activation()
{
    if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
        return false;
    }

    return true;
}

// If WooCommerce is not active, stop loading the plugin
if (!check_woocommerce_activation()) {
    return;
}

define('APPPAGO_MIN_NUMBER_INSTALLMENTS', 2);
define('APPPAGO_MAX_NUMBER_INSTALLMENTS', 24);
// rata minima per cui viene visualizzato il widget
define('APPPAGO_MIN_AMOUNT_INSTALLMENTS', 20);

add_filter('woocommerce_payment_gateways', 'add_woocommerce_gateway');
wp_clear_scheduled_hook('woocommerce_cancel_unpaid_orders');

add_action('plugins_loaded', 'init_woocommerce_gateway_class');

function add_woocommerce_gateway($gateways)
{
    $gateways[] = 'Woocommerce_gateway';
    return $gateways;
}

function custom_gateway_icon($icon, $id)
{
    if ($id == "apppago_gateway") {
        return '<img src="' . plugins_url('apppago_smallcrop.png', __FILE__) . '" style="max-width: 4rem; text-align: center" > ';
    }

}

function init_woocommerce_gateway_class()
{

    class Woocommerce_gateway extends WC_Payment_Gateway
    {

        public function __construct()
        {
            $this->id = 'apppago_gateway';
            $this->description = '';
            $this->title = 'Paga in comode quote con AppPago';
            $this->icon = add_filter('woocommerce_gateway_icon', 'custom_gateway_icon', 10, 2);
            $this->has_fields = true;
            $this->method_title = 'AppPago';
            $this->method_description = 'Paga in comode quote con AppPago';
            $this->supports = array(
                'products'
            );

            $this->init_form_fields();
            $this->init_settings();

            $GLOBALS["apppago_api_key"] = $this->settings["api_key"];
            $GLOBALS["apppago_test_mode"] = $this->settings["test_mode"];

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_filter( 'woocommerce_available_payment_gateways', array( $this, 'filter_gateways_by_product_categories' ) );
        }

        public function filter_gateways_by_product_categories( $available_gateways ) {
            if ( is_admin() ) {
                return $available_gateways; // Non fare nulla nel backend
            }

            if (!isset($available_gateways[$this->id])) {
                return $available_gateways;
            }

            global $woocommerce;

            // escludo il gateway se il totale dell'ordine è inferiore o superiore ai limiti impostati
            $order_total = $woocommerce->cart->total;

            if ($order_total < $available_gateways[$this->id]->settings["min_amount"] || $order_total > $available_gateways[$this->id]->settings["max_amount"]) {
                unset($available_gateways[$this->id]);

                return $available_gateways;
            }

            // Ottieni le categorie escluse dalle impostazioni
            $excluded_categories = $this->get_option( 'product_categories' );

            if ( empty( $excluded_categories ) ) {
                return $available_gateways; // Non fare nulla se non ci sono categorie escluse
            }

            // Verifica se ci sono prodotti nel carrello appartenenti alle categorie escluse
            foreach ( $woocommerce->cart->get_cart() as $cart_item_key => $cart_item ) {
                $product = $cart_item['data'];
                $product_categories = wc_get_product_term_ids( $product->get_id(), 'product_cat' );

                if ( ! empty( array_intersect( $excluded_categories, $product_categories ) ) ) {
                    unset( $available_gateways[$this->id] ); // Rimuovi il gateway se c'è una corrispondenza
                    return $available_gateways;
                }
            }

            return $available_gateways;
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                "test_mode" => array(
                    "title" => __("Modalità di Testing", "woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Abilità Modalità di Testing", "woocommerce"),
                    "default" => "no"
                ),
                "merchant_code" => array(
                    "title" => __("Merchant Code", "woocommerce"),
                    "type" => "text",
                    "description" => __("Inserisci il tuo merchant code", "woocommerce"),
                    "default" => "",
                    "required" => true
                ),
                "api_key" => array(
                    "type" => "hidden",
                    "default" => bin2hex(random_bytes(16))
                ),
                "vat" => array(
                    "title" => __("VAT", "woocommerce"),
                    "type" => "text",
                    "description" => __("Inserisci la tua partita IVA", "woocommerce"),
                    "default" => "",
                    "required" => true
                ),
                "min_amount" => array(
                    "title" => __("Importo Minimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci l'importo minimo", "woocommerce"),
                    "default" => ""
                ),
                "max_amount" => array(
                    "title" => __("Importo Massimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci l'importo massimo", "woocommerce"),
                    "default" => ""
                ),
                "installments_type" => array(
                    "title" => __("Tipo di mensilità", "woocommerce"),
                    "type" => "select",
                    "description" => __("Scegli il tipo di mensilità da offrire", "woocommerce"),
                    "default" => "none",
                    "options" => array(
                        "none" => "Scegli...",
                        "fixed" => "Fisso",
                        "range" => "Range",
                        "select" => "Selezione Multipla"
                    )
                ),
                "fixed_installments_number" => array(
                    "title" => __("Numero di mensilità", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci il numero di mensilità fisso da offrire", "woocommerce"),
                    "default" => ""
                ),
                "range_installments_min" => array(
                    "title" => __("Numero di mensilità Minimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci il numero minimo di mensilità da offrire", "woocommerce"),
                    "default" => ""
                ),
                "range_installments_max" => array(
                    "title" => __("Numero di mensilità Massimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci il numero massimo di mensilità da offrire", "woocommerce"),
                    "default" => ""
                ),
                "select_installments_list" => array(
                    "title" => __("Lista mensilità da Offrire", "woocommerce"),
                    "type" => "multiselect",
                    'class'       => 'wc-enhanced-select',
                    "description" => __("Scegli le mensilità da offrire", "woocommerce"),
                    "default" => array(),
                    "options" => array_combine(
                        range(APPPAGO_MIN_NUMBER_INSTALLMENTS, APPPAGO_MAX_NUMBER_INSTALLMENTS),
                        range(APPPAGO_MIN_NUMBER_INSTALLMENTS, APPPAGO_MAX_NUMBER_INSTALLMENTS)
                    ),
                    'custom_attributes' => array(
                        'data-placeholder' => __( 'Seleziona le mensilità da offrire', 'text-domain' ),
                    ),
                ),
                "iban_allowed" => array(
                    "title" => __("Permetti IBAN", "woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Abilita l'acquisto tramite IBAN", "woocommerce"),
                    "default" => "no"
                ),
                "company_allowed" => array(
                    "title" => __("Permetti Partita IVA", "woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Abilita l'acquisto tramite partita IVA", "woocommerce"),
                    "default" => "no"
                ),
                'product_categories' => array(
                    'title'       => __( 'Categorie prodotti da escludere', 'text-domain' ),
                    'type'        => 'multiselect',
                    'class'       => 'wc-enhanced-select',
                    'css'         => 'width: 400px;', // Aggiungi stile CSS per la larghezza
                    'options'     => $this->get_product_categories_hierarchy(),
                    'default'     => array(),
                    'description' => __( 'Seleziona le categorie da escludere da questo tipo di pagamento.', 'text-domain' ),
                    'custom_attributes' => array(
                        'data-placeholder' => __( 'Seleziona le categorie da escludere', 'text-domain' ),
                    ),
                ),
            );

            add_action('admin_head', function () {

                if (!is_admin())
                    return;

                echo '
                <script>
                    document.addEventListener("DOMContentLoaded", () => {
                        let installmentsType = document.getElementById("woocommerce_apppago_gateway_installments_type");
                        console.log("DOM LOADED");
                        let fixedInstallmentsNumber = document.getElementById(
                            "woocommerce_apppago_gateway_fixed_installments_number"
                        ).parentNode.parentNode.parentNode;

                        let rangeInstallmentsMin = document.getElementById(
                            "woocommerce_apppago_gateway_range_installments_min"
                        ).parentNode.parentNode.parentNode;
                        let rangeInstallmentsMax = document.getElementById(
                            "woocommerce_apppago_gateway_range_installments_max"
                        ).parentNode.parentNode.parentNode;

                        let selectInstallmentsList = document.getElementById(
                            "woocommerce_apppago_gateway_select_installments_list"
                        ).parentNode.parentNode.parentNode;

                        if (installmentsType.value == "fixed") {
                            fixedInstallmentsNumber.style.display = "";
                            rangeInstallmentsMin.style.display = "none";
                            rangeInstallmentsMax.style.display = "none";
                            selectInstallmentsList.style.display = "none";
                        } else if (installmentsType.value == "range") {
                            fixedInstallmentsNumber.style.display = "none";
                            selectInstallmentsList.style.display = "none";
                            rangeInstallmentsMin.style.display = "";
                            rangeInstallmentsMax.style.display = "";
                        } else if (installmentsType.value == "select") {
                            fixedInstallmentsNumber.style.display = "none";
                            rangeInstallmentsMin.style.display = "none";
                            rangeInstallmentsMax.style.display = "none";
                            selectInstallmentsList.style.display = "";
                        } else {
                            fixedInstallmentsNumber.style.display = "none";
                            rangeInstallmentsMin.style.display = "none";
                            rangeInstallmentsMax.style.display = "none";
                            selectInstallmentsList.style.display = "none";
                        }

                        installmentsType.addEventListener("change", (e) => {
                            if (e.target.value == "fixed") {
                                fixedInstallmentsNumber.style.display = "";
                                rangeInstallmentsMin.style.display = "none";
                                rangeInstallmentsMax.style.display = "none";
                                selectInstallmentsList.style.display = "none";
                            } else if (e.target.value == "range") {
                                fixedInstallmentsNumber.style.display = "none";
                                rangeInstallmentsMin.style.display = "";
                                rangeInstallmentsMax.style.display = "";
                                selectInstallmentsList.style.display = "none";
                            } else if (e.target.value == "select") {
                                fixedInstallmentsNumber.style.display = "none";
                                rangeInstallmentsMin.style.display = "none";
                                rangeInstallmentsMax.style.display = "none";
                                selectInstallmentsList.style.display = "";
                            } else {
                                fixedInstallmentsNumber.style.display = "none";
                                rangeInstallmentsMin.style.display = "none";
                                rangeInstallmentsMax.style.display = "none";
                                selectInstallmentsList.style.display = "none";
                            }
                        });
                    });

                </script>';

            });

        }

        private function get_product_categories_hierarchy( $parent = 0, $depth = 0 ) {
            $categories = get_categories( array(
                'taxonomy'     => 'product_cat',
                'orderby'      => 'name',
                'hide_empty'   => false,
                'hierarchical' => true,
                'parent'       => $parent,
            ) );

            $hierarchy = array();
            foreach ( $categories as $category ) {
                $hierarchy[$category->term_id] = str_repeat( '&nbsp;&nbsp;&nbsp;&nbsp;', $depth ) . $category->name; // Usa &nbsp; per l'indentazione
                $hierarchy += $this->get_product_categories_hierarchy( $category->term_id, $depth + 1 );
            }

            return $hierarchy;
        }

        public function process_admin_options()
        {
            global $apppago_base_url_testing, $apppago_base_url_production, $apppago_current_version;

            if(!isset($_POST["woocommerce_apppago_gateway_api_key"]))
                $_POST["woocommerce_apppago_gateway_api_key"] = bin2hex(random_bytes(16));

            $merchant_code = $_POST["woocommerce_apppago_gateway_merchant_code"] ?? "";
            $api_key = $_POST["woocommerce_apppago_gateway_api_key"];
            $vat = $_POST["woocommerce_apppago_gateway_vat"] ?? "";
            $min_amount = $_POST["woocommerce_apppago_gateway_min_amount"] ?? "";
            $max_amount = $_POST["woocommerce_apppago_gateway_max_amount"] ?? "";
            $installmentsType = $_POST["woocommerce_apppago_gateway_installments_type"] ?? "";
            $fixedInstallmentsNumber = $_POST["woocommerce_apppago_gateway_fixed_installments_number"] ?? "";
            $rangeInstallmentsMin = $_POST["woocommerce_apppago_gateway_range_installments_min"] ?? "";
            $rangeInstallmentsMax = $_POST["woocommerce_apppago_gateway_range_installments_max"] ?? "";
            $select_installments_list = $_POST["woocommerce_apppago_gateway_select_installments_list"] ?? [];
            $ibanAllowed = $_POST["woocommerce_apppago_gateway_iban_allowed"] ?? "0";
            $companyAllowed = $_POST["woocommerce_apppago_gateway_company_allowed"] ?? "0";
            $excludedProductCategories = $_POST['woocommerce_apppago_gateway_product_categories'] ?? [];

            if(empty($api_key) || empty($merchant_code) || empty($vat)) {
                WC_Admin_Settings::add_error("Errore di validazione: merchant code e partita iva sono obbligatori.");
                return false;
            }

            $args = array(
                "body" => array(
                    "platform" => "woocommerce",
                    "version" => $apppago_current_version,
                    "host" => get_site_url(),
                    "merchant_code" => $merchant_code,
                    "api_key" => $api_key,
                    "vat" => $vat,
                    "max_amount" => $max_amount,
                    "min_amount" => $min_amount,
                    "installments_type" => $installmentsType,
                    "fixed_installments_number" => $fixedInstallmentsNumber,
                    "range_installments_min" => $rangeInstallmentsMin,
                    "range_installments_max" => $rangeInstallmentsMax,
                    "select_installments_list" => $select_installments_list,
                    "iban_allowed" => $ibanAllowed,
                    "company_allowed" => $companyAllowed
                )
            );

            // $base_url = isset($_POST["woocommerce_apppago_gateway_test_mode"]) && $_POST["woocommerce_apppago_gateway_test_mode"] == 1 ? $apppago_base_url_testing : $apppago_base_url_production;

            $base_url = $apppago_base_url_production;

            $response = wp_remote_post($base_url . "api/config", $args);
            $response_body = wp_remote_retrieve_body($response);

            if (wp_remote_retrieve_response_code($response) == 500) {
                WC_Admin_Settings::add_error("Errore del server. riprovare più tardi");
                return false;
            } else if (wp_remote_retrieve_response_code($response) == 400) {
                WC_Admin_Settings::add_error("Errore di validazione del merchant Id. Controllare i dati inseriti");
                return false;
            } else if (wp_remote_retrieve_response_code($response) == 200) {
                parent::process_admin_options();
                return true;
            } else {
                WC_Admin_Settings::add_error("Errore sconosciuto. Controllare i dati inseriti");
                return false;
            }
        }

        public function payment_fields()
        {

            //SHOW A DESCRIPTION
            if ($this->description) {
                echo wpautop(wp_kses_post("$this->description"));
            }

            echo '<div class="form-row form-row-wide">
                <input type="hidden" name="clientType" value="privato">
                <label>Codice Fiscale:</label>
                <input id="taxCode" class="input-text" style="font-size: 1.5em;padding: 8px; background-repeat: no-repeat; background-position: right .618em center; background-size: 32px 20px;" type="text" autocomplete="off" placeholder="Inserisci il tuo codice fiscale" name="taxCode" />
                </div>
                <div class="clear"></div>';

            echo '<div class="clear"></div></fieldset>';

        }

        public function payment_scripts()
        {
        }

        public function process_payment($order_id)
        {
            global $apppago_base_url_testing, $apppago_base_url_production, $woocommerce;
            $base_url = $this->settings["test_mode"] == "yes" ? $apppago_base_url_testing : $apppago_base_url_production;

            $order = new WC_Order($order_id);
            $order->update_status('failed', __('L\'utente è nel flow di checkout', 'woocommerce'));

            if (!isset($_POST["clientType"])) {
                wc_add_notice("Selezionare il tipo di cliente", "error");
                return array(
                    'result' => 'fail'
                );
            }
            if ($_POST["clientType"] == "privato" && !preg_match("/^[A-Z]{6}[0-9]{2}[A-Z][0-9]{2}[A-Z][0-9]{3}[A-Z]$/", strtoupper($_POST["taxCode"]))) {
                wc_add_notice("Il codice fiscale inserito non è valido", "error");
                return array(
                    'result' => 'fail'
                );

            } else if ($_POST["clientType"] == "azienda" && !preg_match("/^[0-9]{11}$/", strtoupper($_POST["taxCode"]))) {
                wc_add_notice("La partita IVA inserita non è valida", "error");
                return array(
                    'result' => 'fail'
                );
            }

            $olMerchantTrxId = substr(str_shuffle("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"), 0, 5);

            try {
                $body = array(
                    "olMerchantTrxId" => $olMerchantTrxId,
                    "merchantId" => $this->settings["merchant_code"],
                    "loanTotalAmount" => $order->get_total(),
                    "installmentNumber" => $this->settings["installments_type"] == "fixed" ? $this->settings["fixed_installments_number"] : "",
                    "firstname" => $order->get_billing_first_name(),
                    "lastname" => $order->get_billing_last_name(),
                    "taxCode" => strtoupper($_POST["taxCode"]),
                    "residenceAddress" => $order->get_billing_address_1(),
                    "residenceCity" => $order->get_billing_city(),
                    "residenceState" => $order->get_billing_state(),
                    "residencePostalCode" => $order->get_billing_postcode(),
                    "email" => $order->get_billing_email(),
                    "phone" => $order->get_billing_phone(),
                    "merchantRedirect" => get_site_url() . "/wp-json/apppago/callback/",
                    "merchant_order_id" => $order_id,
                );

                $url = wp_remote_post(
                    $base_url . "api/order",
                    array(
                        'body' => $body
                    )
                );
                $url = wp_remote_retrieve_body($url);

                $url = json_decode($url, true);


                if ($url["redirectUrl"]) {
                    $order->update_status('on-hold', __('Flusso AppPago: in attesa che l\'utente completi il flusso', 'woocommerce'));
                    return array(
                        'result' => 'success',
                        'redirect' => $url["redirectUrl"]
                    );
                } else {
                    $order->update_status('failed', __('Ordine annullato, impossibile contattare server AppPago', 'woocommerce'));
                    wc_add_notice("Errore sconosciuto. Riprovare più tardi", "error");
                    return array(
                        'result' => 'fail'
                    );
                }
            } catch (Exception $e) {
                $order->update_status('failed', __('Ordine annullato, impossibile contattare server AppPago', 'woocommerce'));
                wc_add_notice("Errore sconosciuto. Riprovare più tardi", "error");
                return;
            }
        }
    }
}